@@ -17,7 +17,6 @@
 */
 #include "config.h"
 
-#include "glib-compat.h"
 #include "spice-client.h"
 #include "spice-common.h"
 
@@ -32,7 +31,7 @@
  * @section_id:
  * @see_also: #SpiceChannel, and the GTK widget #SpiceDisplay
  * @stability: Stable
- * @include: channel-cursor.h
+ * @include: spice-client.h
  *
  * The Spice protocol defines a set of messages for controlling cursor
  * shape and position on the remote display area. The cursor changes
@@ -91,7 +90,7 @@ static void spice_cursor_channel_finalize(GObject *obj)
     SpiceCursorChannel *channel = SPICE_CURSOR_CHANNEL(obj);
     SpiceCursorChannelPrivate *c = channel->priv;
 
-    g_clear_pointer(&c->cursors, cache_unref);
+    g_clear_pointer(&c->cursors, cache_free);
 
     if (G_OBJECT_CLASS(spice_cursor_channel_parent_class)->finalize)
         G_OBJECT_CLASS(spice_cursor_channel_parent_class)->finalize(obj);
@@ -305,7 +304,7 @@ static display_cursor *set_cursor(SpiceChannel *channel, SpiceCursor *scursor)
     guint8 *rgba;
     guint8 val;
 
-    CHANNEL_DEBUG(channel, "%s: flags %d, size %d", __FUNCTION__,
+    CHANNEL_DEBUG(channel, "%s: flags %x, size %u", __FUNCTION__,
                   scursor->flags, scursor->data_size);
 
     if (scursor->flags & SPICE_CURSOR_FLAGS_NONE)
