@@ -97,6 +97,8 @@ enum {
     SPICE_DISPLAY_LAST_SIGNAL,
 };
 
+#define DEFAULT_KEYPRESS_DELAY 100
+
 static guint signals[SPICE_DISPLAY_LAST_SIGNAL];
 
 #ifdef G_OS_WIN32
@@ -329,6 +331,22 @@ whole:
     set_monitor_ready(display, true);
 }
 
+static void
+spice_display_set_keypress_delay(SpiceDisplay *display, guint delay)
+{
+    SpiceDisplayPrivate *d = display->priv;
+    const gchar *env = g_getenv("SPICE_KEYPRESS_DELAY");
+
+    if (env != NULL)
+        delay = strtoul(env, NULL, 10);
+
+    if (d->keypress_delay != delay) {
+        SPICE_DEBUG("keypress-delay is set to %u ms", delay);
+        d->keypress_delay = delay;
+        g_object_notify(G_OBJECT(display), "keypress-delay");
+    }
+}
+
 static void spice_display_set_property(GObject      *object,
                                        guint         prop_id,
                                        const GValue *value,
@@ -386,15 +404,7 @@ static void spice_display_set_property(GObject      *object,
         scaling_updated(display);
         break;
     case PROP_KEYPRESS_DELAY:
-        {
-            const gchar *env = g_getenv("SPICE_KEYPRESS_DELAY");
-            guint delay = g_value_get_uint(value);
-            if (env != NULL)
-                delay = strtoul(env, NULL, 10);
-
-            SPICE_DEBUG("keypress-delay is set to %u ms", delay);
-            d->keypress_delay = delay;
-        }
+        spice_display_set_keypress_delay(display, g_value_get_uint(value));
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -462,8 +472,19 @@ static GdkCursor* get_blank_cursor(void)
 static gboolean grab_broken(SpiceDisplay *self, GdkEventGrabBroken *event,
                             gpointer user_data G_GNUC_UNUSED)
 {
+    GdkWindow *window = gtk_widget_get_window(GTK_WIDGET(self));
     SPICE_DEBUG("%s (implicit: %d, keyboard: %d)", __FUNCTION__,
                 event->implicit, event->keyboard);
+    SPICE_DEBUG("%s (SpiceDisplay::GdkWindow %p, event->grab_window: %p)",
+                __FUNCTION__, window, event->grab_window);
+    if (window == event->grab_window) {
+        /* ignore grab-broken event moving the grab to GtkEventBox::window
+         * (from GtkEventBox::event_window) as we initially called
+         * gdk_pointer_grab() on GtkEventBox::window, see
+         * https://bugzilla.gnome.org/show_bug.cgi?id=769635
+         */
+        return false;
+    }
 
     if (event->keyboard) {
         try_keyboard_ungrab(self);
@@ -478,6 +499,24 @@ static gboolean grab_broken(SpiceDisplay *self, GdkEventGrabBroken *event,
     return false;
 }
 
+static void file_transfer_callback(GObject *source_object,
+                                   GAsyncResult *result,
+                                   gpointer user_data G_GNUC_UNUSED)
+{
+    SpiceMainChannel *channel = SPICE_MAIN_CHANNEL(source_object);
+    GError *error = NULL;
+
+    if (spice_main_file_copy_finish(channel, result, &error))
+        return;
+
+    if (error != NULL && error->message != NULL)
+        g_warning("File transfer failed with error: %s", error->message);
+    else
+        g_warning("File transfer failed");
+
+    g_clear_error(&error);
+}
+
 static void drag_data_received_callback(SpiceDisplay *self,
                                         GdkDragContext *drag_context,
                                         gint x,
@@ -510,7 +549,7 @@ static void drag_data_received_callback(SpiceDisplay *self,
     g_strfreev(file_urls);
 
     spice_main_file_copy_async(d->main, files, 0, NULL, NULL,
-                               NULL, NULL, NULL);
+                               NULL, file_transfer_callback, NULL);
     for (i = 0; i < n_files; i++) {
         g_object_unref(files[i]);
     }
@@ -568,20 +607,11 @@ drawing_area_realize(GtkWidget *area, gpointer user_data)
 {
 #ifdef GDK_WINDOWING_X11
     SpiceDisplay *display = SPICE_DISPLAY(user_data);
-    GError *err = NULL;
 
-    if (!GDK_IS_X11_DISPLAY(gdk_display_get_default()))
-        return;
+    if (GDK_IS_X11_DISPLAY(gdk_display_get_default()) &&
+        spice_display_get_gl_scanout(SPICE_DISPLAY_CHANNEL(display->priv->display)) != NULL)
+        spice_display_widget_gl_scanout(display);
 
-    if (!spice_egl_init(display, &err)) {
-        g_critical("egl init failed: %s", err->message);
-        g_clear_error(&err);
-    }
-
-    if (!spice_egl_realize_display(display, gtk_widget_get_window(area), &err)) {
-        g_critical("egl realize failed: %s", err->message);
-        g_clear_error(&err);
-    }
 #endif
 }
 
@@ -1195,7 +1225,6 @@ static gboolean do_color_convert(SpiceDisplay *display, GdkRectangle *r)
 static void set_egl_enabled(SpiceDisplay *display, bool enabled)
 {
     SpiceDisplayPrivate *d = display->priv;
-    GtkWidget *area;
 
     if (egl_enabled(d) == enabled)
         return;
@@ -1205,12 +1234,11 @@ static void set_egl_enabled(SpiceDisplay *display, bool enabled)
         /* even though the function is marked as deprecated, it's the
          * only way I found to prevent glitches when the window is
          * resized. */
-        area = gtk_stack_get_child_by_name(d->stack, "draw-area");
+        GtkWidget *area = gtk_stack_get_child_by_name(d->stack, "draw-area");
         gtk_widget_set_double_buffered(GTK_WIDGET(area), !enabled);
     } else
 #endif
     {
-        area = gtk_stack_get_child_by_name(d->stack, "gl-area");
         gtk_stack_set_visible_child_name(d->stack,
                                          enabled ? "gl-area" : "draw-area");
     }
@@ -1288,6 +1316,25 @@ static gboolean key_press_delayed(gpointer data)
     return FALSE;
 }
 
+static bool send_pause(SpiceDisplay *display, GdkEventType type)
+{
+    SpiceInputsChannel *inputs = display->priv->inputs;
+
+    /* Send proper scancodes. This will send same scancodes
+     * as hardware.
+     * The 0x21d is a sort of Third-Ctrl while
+     * 0x45 is the NumLock.
+     */
+    if (type == GDK_KEY_PRESS) {
+        spice_inputs_key_press(inputs, 0x21d);
+        spice_inputs_key_press(inputs, 0x45);
+    } else {
+        spice_inputs_key_release(inputs, 0x21d);
+        spice_inputs_key_release(inputs, 0x45);
+    }
+    return true;
+}
+
 static void send_key(SpiceDisplay *display, int scancode, SendKeyType type, gboolean press_delayed)
 {
     SpiceDisplayPrivate *d = display->priv;
@@ -1409,7 +1456,9 @@ static gboolean check_for_grab_key_released(SpiceDisplay *display, int type, int
 static void update_display(SpiceDisplay *display)
 {
 #ifdef G_OS_WIN32
-    win32_window = display ? GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(display))) : NULL;
+    win32_window = display ?
+                        gdk_win32_window_get_impl_hwnd(gtk_widget_get_window(GTK_WIDGET(display))) :
+                        NULL;
 #endif
 }
 
@@ -1481,6 +1530,16 @@ static gboolean key_event(GtkWidget *widget, GdkEventKey *key)
     if (!d->inputs)
         return true;
 
+    if (key->keyval == GDK_KEY_Pause
+#ifdef G_OS_WIN32
+        /* for some reason GDK does not fill keyval for VK_PAUSE 
+         * See https://bugzilla.gnome.org/show_bug.cgi?id=769214
+         */
+        || key->hardware_keycode == VK_PAUSE
+#endif
+        ) {
+        return send_pause(display, key->type);
+    }
     if (!scancode)
         scancode = vnc_display_keymap_gdk2xtkbd(d->keycode_map, d->keycode_maplen,
                                                 key->hardware_keycode);
@@ -2153,7 +2212,7 @@ static void spice_display_class_init(SpiceDisplayClass *klass)
         (gobject_class, PROP_KEYPRESS_DELAY,
          g_param_spec_uint("keypress-delay", "Keypress delay",
                            "Keypress delay",
-                           0, G_MAXUINT, 100,
+                           0, G_MAXUINT, DEFAULT_KEYPRESS_DELAY,
                            G_PARAM_READWRITE |
                            G_PARAM_CONSTRUCT |
                            G_PARAM_STATIC_STRINGS));
@@ -2630,18 +2689,56 @@ static void cursor_reset(SpiceCursorChannel *channel, gpointer data)
     gdk_window_set_cursor(window, NULL);
 }
 
+static void inputs_channel_event(SpiceChannel *channel, SpiceChannelEvent event,
+                                 gpointer data)
+{
+    SpiceDisplay *display = data;
+    guint delay = DEFAULT_KEYPRESS_DELAY;
+    GSocket *sock;
+
+    if (event != SPICE_CHANNEL_OPENED)
+        return;
+
+    g_object_get(channel, "socket", &sock, NULL);
+    if (g_socket_get_family(sock) == G_SOCKET_FAMILY_UNIX) {
+        delay = 0;
+    }
+    g_object_unref(sock);
+
+    spice_display_set_keypress_delay(display, delay);
+}
+
 #ifndef G_OS_WIN32
 G_GNUC_INTERNAL
 void spice_display_widget_gl_scanout(SpiceDisplay *display)
 {
     SpiceDisplayPrivate *d = display->priv;
+    GError *err = NULL;
 
     SPICE_DEBUG("%s: got scanout",  __FUNCTION__);
+
+#ifdef GDK_WINDOWING_X11
+    GtkWidget *area = gtk_stack_get_child_by_name(d->stack, "draw-area");
+
+    if (GDK_IS_X11_DISPLAY(gdk_display_get_default()) &&
+        !d->egl.context_ready &&
+        gtk_widget_get_realized(area)) {
+        if (!spice_egl_init(display, &err)) {
+            g_critical("egl init failed: %s", err->message);
+            g_clear_error(&err);
+        }
+
+        if (!spice_egl_realize_display(display, gtk_widget_get_window(area), &err)) {
+            g_critical("egl realize failed: %s", err->message);
+            g_clear_error(&err);
+        }
+    }
+#endif
+
     set_egl_enabled(display, true);
 
     if (d->egl.context_ready) {
         const SpiceGlScanout *scanout;
-        GError *err = NULL;
 
         scanout = spice_display_get_gl_scanout(SPICE_DISPLAY_CHANNEL(d->display));
         /* should only be called when the display has a scanout */
@@ -2748,6 +2845,8 @@ static void channel_new(SpiceSession *s, SpiceChannel *channel, gpointer data)
     if (SPICE_IS_INPUTS_CHANNEL(channel)) {
         d->inputs = SPICE_INPUTS_CHANNEL(channel);
         spice_channel_connect(channel);
+        spice_g_signal_connect_object(channel, "channel-event",
+                                      G_CALLBACK(inputs_channel_event), display, 0);
         return;
     }
 
