@@ -745,7 +745,7 @@ static const GtkToggleActionEntry tentries[] = {
         .callback    = G_CALLBACK(menu_cb_bool_prop),
     },{
         .name        = "grab-mouse",
-        .label       = "Grab mouse in server mode (no tabled/vdagent)",
+        .label       = "Grab mouse in server mode (no tablet/vdagent)",
         .callback    = G_CALLBACK(menu_cb_bool_prop),
     },{
         .name        = "resize-guest",
@@ -1409,6 +1409,8 @@ static gboolean input_cb(GIOChannel *gin, GIOCondition condition, gpointer data)
     return TRUE;
 }
 
+static void watch_stdin(void);
+
 static void port_opened(SpiceChannel *channel, GParamSpec *pspec,
                         spice_connection *conn)
 {
@@ -1432,6 +1434,7 @@ static void port_opened(SpiceChannel *channel, GParamSpec *pspec,
 
         /* handle the first spicy port and connect it to stdin/out */
         if (g_strcmp0(name, "org.spice.spicy") == 0 && stdin_port == NULL) {
+            watch_stdin();
             stdin_port = port;
         }
     } else {
@@ -1514,6 +1517,7 @@ task_cancel_cb(GtkButton *button,
 static TransferTaskWidgets *
 transfer_task_widgets_new(SpiceFileTransferTask *task)
 {
+    char *filename;
     TransferTaskWidgets *widgets = g_new0(TransferTaskWidgets, 1);
 
     widgets->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
@@ -1521,7 +1525,9 @@ transfer_task_widgets_new(SpiceFileTransferTask *task)
     widgets->cancel = gtk_button_new_with_label("Cancel");
 
     widgets->progress = gtk_progress_bar_new();
-    widgets->label = gtk_label_new(spice_file_transfer_task_get_filename(task));
+    filename = spice_file_transfer_task_get_filename(task);
+    widgets->label = gtk_label_new(filename);
+    g_free(filename);
 
     gtk_widget_set_halign(widgets->label, GTK_ALIGN_START);
     gtk_widget_set_valign(widgets->label, GTK_ALIGN_BASELINE);
@@ -1812,14 +1818,18 @@ static void setup_terminal(gboolean reset)
         return;
 
 #ifdef HAVE_TERMIOS_H
-    static struct termios saved_tios;
     struct termios tios;
+    static struct termios saved_tios;
+    static bool saved = false;
 
-    if (reset)
+    if (reset) {
+        if (!saved)
+            return;
         tios = saved_tios;
-    else {
+    } else {
         tcgetattr(stdinfd, &tios);
         saved_tios = tios;
+        saved = true;
         tios.c_lflag &= ~(ICANON | ECHO);
     }
 
@@ -1905,8 +1915,6 @@ int main(int argc, char *argv[])
     g_free(tls_port);
     g_free(unix_path);
 
-    watch_stdin();
-
     connection_connect(conn);
     if (connections > 0)
         g_main_loop_run(mainloop);
